﻿/**
* @file StovePCEnum.h
* @brief StovePCEnum 헤더파일
* @details StovePCSDK 사용하는 enum을 선언합니다.
*/
#ifndef STOVEPCENUM_H_
#define STOVEPCENUM_H_


/*!
*  @struct StovePCResult
*  @brief 스토브에서 정의한 API 호출결과에 대한 enum 입니다.
*  @brief API 호출시 API 호출 후 결과 상태에 따라 반환됩니다.
*/
enum StovePCResult
{
    /// 정상 
    STOVE_PC_NO_ERROR = 0,

    // Config & Env Error  50~99
    /// 설정된 환경값이 유효하지 않음
    STOVE_PC_INVALID_CONFIG = 50,

    /// 설정된 콜백이 유효하지 않음 
    STOVE_PC_INVALID_CALLBACK = 51,

    /// 설정된 로그레벨의 값이 유효하지 않음
    STOVE_PC_INVALID_LOG_LEVEL = 52,

    /// 설정된 로그경로가 유효하지 않음 
    STOVE_PC_INVALID_LOG_PATH = 53,

    /// 서버로부터 환경정보를 받지 못함 
    STOVE_PC_INVALID_ENV = 54,

    /// 설정된 파라미터가 유효하지 않음
    STOVE_PC_INVALID_PARAM = 55,

    //Core Error          100 ~149
    /// StovePCSDK 가 초기화 되지 않음  
    STOVE_PC_CORE_NOT_INITIALIZE = 100,

    /// StovePCSDK 가 이미 초기화 됨 
    STOVE_PC_CORE_ALREADY_INITIALIZE = 101,

    /// 내부의 모든 Task가 비어 있음
    STOVE_PC_CORE_TASKS_ALL_EMPTY = 102,

    /// 콜백 호출시 콜백의 주소가 없음 
    STOVE_PC_CORE_CALLBACK_EXECUTE_NULLPTR = 103,

    //SGUP Error 150 ~ 199
    ///  SDK 내부의 SGUP TASK가 초기화 되지 않음 
    STOVE_PC_SGUP_NOT_INITIALIZE = 150,

    /// SGUP Task가 유효하지 않음
    STOVE_PC_SGUP_TASK_EMPTY = 151,

    /// SGUP Task에서 외부에러가 발생함[deprecated]
    STOVE_PC_SGUP_EXTERNAL_ERROR = 152,

    /// SGUP OnToken 콜백이 없음 
    STOVE_PC_SGUP_ONTOKEN_EMPTY = 153,

    /// SGUP OnUser 콜백이 없음  
    STOVE_PC_SGUP_ONUSER_EMPTY = 154,

    /// SGUP Token이 유효하지 않음 
    STOVE_PC_SGUP_TOKEN_INVALID = 155,

    /// SDK 내부의 CommandTask가 초기화 되지 않음 
    STOVE_PC_COMMAND_NOT_INITIALIZE = 156,

    /// SDK 내부의 EnvTask가 초기화 되지 않음 
    STOVE_PC_ENV_NOT_INITIALIZE = 157,

    /// Command Token이 유효하지 않음
    STOVE_PC_COMMAND_TOKEN_INVALID = 158,

    /// Env Token이 유효하지 않음
    STOVE_PC_ENV_TOKEN_INVALID = 159,

    //OwnerShip Error 200 ~ 249
    /// Ownership Task가 초기화 되지 않음[deprecated]
    STOVE_PC_OWNERSHIP_NOT_INITIALIZE = 200,

    /// Ownership 조회시 토큰이 유효하지 않음
    STOVE_PC_OWNERSHIP_TOKEN_INVALID = 201,

    /// Ownership Task가 비어있음 
    STOVE_PC_OWNERSHIP_TASK_EMPTY = 202,

    /// Ownership OnOwnership 콜백이 비어 있음 
    STOVE_PC_OWNERSHIP_ONOWNERSHIP_EMPTY = 203,

    /// Ownership 조회시 HTTP Error 발생 
    STOVE_PC_OWNERSHIP_HTTP_ERROR = 204,

    /// Ownership 조회 후 응답코드 비정상
    STOVE_PC_OWNERSHIP_RESPONSE_ERROR = 205,

    //Stats Error 250 ~ 299
    /// Stats 기능 호출시 파라미터가 유효하지 않음
    STOVE_PC_STATS_PARAM_INVALID = 250,

    /// Stats 조회시 토큰이 유효하지 않음
    STOVE_PC_STATS_TOKEN_INVALID = 251,

    /// Stats Task가 비어있음 
    STOVE_PC_STATS_TASK_EMPTY = 252,

    /// Stats 콜백이 비어 있음 
    STOVE_PC_STATS_ONSTAT_EMPTY = 253,

    /// Stats 콜백이 비어 있음 
    STOVE_PC_STATS_ONSETSTAT_EMPTY = 254,

    /// Stats 조회시 HTTP Error 발생 
    STOVE_PC_STATS_HTTP_ERROR = 255,

    /// Stats 조회 후 응답코드 비정상
    STOVE_PC_STATS_RESPONSE_ERROR = 256,

    //Achievement Error 300 ~ 349
    /// Achievement 기능 호출시 파라미터가 유효하지 않음
    STOVE_PC_ACHIEVEMENT_PARAM_INVALID = 300,

    /// Achievement 조회시 토큰이 유효하지 않음
    STOVE_PC_ACHIEVEMENT_TOKEN_INVALID = 301,

    /// Achievement Task가 비어있음 
    STOVE_PC_ACHIEVEMENT_TASK_EMPTY = 302,

    /// Achievement 콜백이 비어 있음 
    STOVE_PC_ACHIEVEMENT_ONACHIEVEMENT_EMPTY = 303,

    /// Achievement 콜백이 비어 있음 
    STOVE_PC_ACHIEVEMENT_ONALLACHIEVEMENT_EMPTY = 304,

    /// Achievement 조회시 HTTP Error 발생 
    STOVE_PC_ACHIEVEMENT_HTTP_ERROR = 305,

    /// Achievement 조회 후 응답코드 비정상
    STOVE_PC_ACHIEVEMENT_RESPONSE_ERROR = 306,

    //Leaderboard Error 350 ~ 399
    /// Leaderboard 기능 호출시 파라미터가 유효하지 않음
    STOVE_PC_LEADERBOARD_PARAM_INVALID = 350,

    /// Leaderboard 조회시 토큰이 유효하지 않음
    STOVE_PC_LEADERBOARD_TOKEN_INVALID = 351,

    /// Leaderboard Task가 비어있음 
    STOVE_PC_LEADERBOARD_TASK_EMPTY = 352,

    /// Leaderboard 콜백이 비어 있음 
    STOVE_PC_LEADERBOARD_ONRANK_EMPTY = 353,

    /// Leaderboard 조회시 HTTP Error 발생 
    STOVE_PC_LEADERBOARD_HTTP_ERROR = 354,

    /// Leaderboard 조회 후 응답코드 비정상
    STOVE_PC_LEADERBOARD_RESPONSE_ERROR = 355,

    //Member Error 400 ~ 449
    /// Member 조회시 토큰이 유효하지 않음
    STOVE_PC_MEMBER_TOKEN_INVALID = 400,

    /// Member Task가 비어있음 
    STOVE_PC_MEMBER_TASK_EMPTY = 401,

    /// Member 조회시 HTTP Error 발생 
    STOVE_PC_MEMBER_HTTP_ERROR = 402,

    /// Member 조회 후 응답코드 비정상
    STOVE_PC_MEMBER_RESPONSE_ERROR = 403,

    //Popup Error 450 ~ 499
    /// Popup 기능 호출시 파라미터가 유효하지 않음
    STOVE_PC_POPUP_PARAM_INVALID = 450,

    /// Popup 조회시 토큰이 유효하지 않음
    STOVE_PC_POPUP_TOKEN_INVALID = 451,

    /// Popup Task가 비어있음 
    STOVE_PC_POPUP_TASK_EMPTY = 452,

    /// Popup 콜백이 비어 있음 
    STOVE_PC_POPUP_ONAUTOPOPUP_EMPTY = 453,

    /// Popup 콜백이 비어 있음 
    STOVE_PC_POPUP_ONMANUALPOPUP_EMPTY = 454,

    /// Popup 콜백이 비어 있음 
    STOVE_PC_POPUP_ONNEWSPOPUP_EMPTY = 455,

    /// Popup 콜백이 비어 있음 
    STOVE_PC_POPUP_ONCOUPONPOPUP_EMPTY = 456,

    /// Popup 콜백이 비어 있음 
    STOVE_PC_POPUP_ONCOMMUNITYPOPUP_EMPTY = 457,

    /// Popup 조회시 HTTP Error 발생 
    STOVE_PC_POPUP_HTTP_ERROR = 458,

    /// Popup 조회 후 응답코드 비정상
    STOVE_PC_POPUP_RESPONSE_ERROR = 459,

    /// SetGameProfile API 호출 하지 않음
    STOVE_PC_POPUP_SETGAMEPROFILE_NOT_CALL = 460,

    //CustomLog 500 ~ 549
    /// CustomLog 기능 호출시 파라미터가 유효하지 않음
    STOVE_PC_CUSTOMLOG_PARAM_INVALID = 500,

    /// CustomLog Task가 비어있음
    STOVE_PC_CUSTOMLOG_TASK_EMPTY = 501,

    /// CustomLog 콜백이 비어 있음 
    STOVE_PC_CUSTOMLOG_ONSTASHCUSTOMEVENT_EMPTY = 502,

    /// CustomLog 조회시 HTTP Error 발생 
    STOVE_PC_CUSTOMLOG_HTTP_ERROR = 503,

    /// CustomLog 조회 후 응답코드 비정상
    STOVE_PC_CUSTOMLOG_RESPONSE_ERROR = 504,

    //Billing 550 ~ 599
    /// Billing 서비스 호출시 파라미터 유효하지 않음
    STOVE_PC_BILLING_PARAM_INVALID = 550,

    /// Billing Task가 비어 있음 
    STOVE_PC_BILLING_TASK_EMPTY = 551,

    /// Billing 서비스를 사용하기 위한 ShopKey 설정이 안됨
    STOVE_PC_BILLING_NOT_SET_SHOP_KEY = 552,

    /// Billing 콜백이 비어 있음 
    STOVE_PC_BILLING_ONFETCHSHOPCATEGORIES_EMPTY = 553,

    /// Billing 콜백이 비어 있음 
    STOVE_PC_BILLING_ONFETCHPRODUCTS_EMPTY = 554,

    /// Billing 콜백이 비어 있음 
    STOVE_PC_BILLING_ONSTARTPURCHASE_EMPTY = 555,

    /// Billing 콜백이 비어 있음 
    STOVE_PC_BILLING_ONCONFIRMPURCHASE_EMPTY = 556,

    /// Billing 콜백이 비어 있음 
    STOVE_PC_BILLING_ONFETCHINVENTORY_EMPTY = 557,

    /// Billing 조회시 HTTP Error 발생 
    STOVE_PC_BILLING_HTTP_ERROR = 558,

    /// Billing 조회 후 응답코드 비정상
    STOVE_PC_BILLING_RESPONSE_ERROR = 559,

    //GlobalDomain Error 600 ~ 649
    /// GlobalDomain 조회시 토큰이 유효하지 않음
    STOVE_PC_GLOBALDOMAIN_TOKEN_INVALID = 600,

    /// GlobalDomain Task가 비어있음 
    STOVE_PC_GLOBALDOMAIN_TASK_EMPTY = 601,

    /// GlobalDomain 조회시 HTTP Error 발생 
    STOVE_PC_GLOBALDOMAIN_HTTP_ERROR = 602,

    /// GlobalDomain 조회 후 응답코드 비정상
    STOVE_PC_GLOBALDOMAIN_RESPONSE_ERROR = 603,

    //Shutdown Error 650 ~ 699
    /// Shutdown 조회시 토큰이 유효하지 않음
    STOVE_PC_SHUTDOWN_TOKEN_INVALID = 650,

    /// Shutdown Task가 비어있음 
    STOVE_PC_SHUTDOWN_TASK_EMPTY = 651,

    /// Shutdown[deprecated] 
    STOVE_PC_SHUTDOWN_TIME = 652,

    /// Shutdown 조회시 HTTP Error 발생 
    STOVE_PC_SHUTDOWN_HTTP_ERROR = 653,

    /// Shutdown 조회 후 응답코드 비정상
    STOVE_PC_SHUTDOWN_RESPONSE_ERROR = 654,

    //Terms Error 700 ~ 749
    /// Terms 조회시 토큰이 유효하지 않음
    STOVE_PC_TERMS_TOKEN_INVALID = 700,

    /// Terms Task가 비어있음 
    STOVE_PC_TERMS_TASK_EMPTY = 701,

    /// Terms 콜백이 비어 있음 
    STOVE_PC_TERMS_ONFETCHTERMSAGREEMENT_EMPTY = 702,

    /// Terms 조회시 HTTP Error 발생 
    STOVE_PC_TERMS_HTTP_ERROR = 703,

    /// Terms 조회 후 응답코드 비정상
    STOVE_PC_TERMS_RESPONSE_ERROR = 704,

    // Exception 900~
    /// StoveSDK  내부에서 Exception 발생
    STOVE_PC_UNMANAGED_EXCEPTION = 900,

    /// StoveSDK 외부에서 Exception 발생 
    STOVE_PC_MANAGED_EXCEPTION = 901

};

/*!
*  @struct StovePCLogLevel
*  @brief 로그레벨 설정을 위한 enum 타입입니다.
*/
enum StovePCLogLevel
{
    /// 로그 파일을 생성하지 않음
    STOVE_PC_LOG_LEVEL_NONE = 1,

    /// 로그레벨 Error 설정
    STOVE_PC_LOG_LEVEL_ERROR = 2,

    /// 로그레벨 Info 설정
    STOVE_PC_LOG_LEVEL_INFO = 3,

    /// 로그레벨 Debug 설정
    STOVE_PC_LOG_LEVEL_DEBUG = 4
};

/*!
*  @struct StovePCInitState
*  @brief StovePC_Init을 호출한 후의 sdk 내부 상태를 나타냅니다.
*/
enum StovePCInitState
{
    /// StovePC_Init이 호출되지 않은 상태(초깃값)
    STOVE_PC_INIT_NONE = 0,

    /// StovePC_Init이 호출된 후, 초기화 진행 중
    STOVE_PC_INIT_PENDING = 1,

    /// StovePC_Init이 호출된 후, 초기화 실패.(OnError 콜백이 호출됩니다.)
    STOVE_PC_INIT_FAIL = 2,

    /// StovePC_Init이 호출된 후, 초기화 완료.(OnInitComplete 콜백이 호출됩니다.)
    STOVE_PC_INIT_COMPLETE = 3,
};

/*!
*  @struct StovePCFunctionType
*  @brief OnError 콜백에서 어떤 함수가 호출됐는지 구분하기 위해 사용되는 enum 타입입니다.
*/
enum StovePCFunctionType
{
    /// StovePC_Init 함수
    STOVE_PC_INIT = 0,

    /// StovePC_GetToken 함수
    STOVE_PC_GET_TOKEN = 1,

    /// StovePC_GetUser 함수
    STOVE_PC_GET_USER = 2,

    /// StovePC_GetOwnership 함수
    STOVE_PC_GET_OWNERSHIP = 3,

    /// StovePC_GetStat 함수
    STOVE_PC_GET_STAT = 4,

    /// StovePC_SetStat 함수
    STOVE_PC_SET_STAT = 5,

    /// StovePC_GetAchievement 함수
    STOVE_PC_GET_ACHIEVEMENT = 6,

    /// StovePC_GetAllAchievement 함수
    STOVE_PC_GET_ALL_ACHIEVEMENT = 7,

    /// StovePC_GetRank 함수
    STOVE_PC_GET_RANK = 8,

    /// StovePC_GetAutoPopup 함수
    STOVE_PC_GET_AUTO_POPUP = 9,

    /// StovePC_GetManualPopup 함수
    STOVE_PC_GET_MANUAL_POPUP = 10,

    /// StovePC_GetNewsPopup 함수
    STOVE_PC_GET_NEWS_POPUP = 11,

    /// StovePC_GetCouponPopup 함수
    STOVE_PC_GET_COUPON_POPUP = 12,

    /// StovePC_GetCommunityPopup 함수
    STOVE_PC_GET_COMMUNITY_POPUP = 13,

    /// StovePC_StashCustomEvent 함수
    STOVE_PC_STASH_CUSTOM_EVENT = 14,

    /// StovePC_IAPInit 함수
    STOVE_PC_IAP_INIT = 15,

    /// StovePC_FetchShopCategories 함수
    STOVE_PC_FETCH_SHOP_CATEGORIES = 16,
 
    /// StovePC_FetchProducts 함수
    STOVE_PC_FETCH_PRODUCTS = 17,

    /// StovePC_StartPurchase 함수
    STOVE_PC_START_PURCHASE = 18,

    /// StovePC_ConfirmPurchase 함수
    STOVE_PC_CONFIRM_PURCHASE = 19,

    /// StovePC_VerifyPurchase 함수
    STOVE_PC_VERIFY_PURCHASE = 20,

    /// StovePC_FetchInventory 함수
    STOVE_PC_FETCH_INVENTORY = 21,

    /// STovePC_FetchTermsAgreement 함수 
    STOVE_PC_FETCH_TERMS_AGREEMENT = 22
};

#endif