﻿#include "StoveSDKPopup.h"

#include <algorithm>

#include "StoveSDKPlugin.h"
#include "Engine.h"
#include "Interfaces/IPluginManager.h"
#include "Slate/DeferredCleanupSlateBrush.h"
#include "Widgets/Images/SThrobber.h"
#include "Runtime/Online/HTTP/Public/Http.h"
#include "Blueprint/WidgetBlueprintLibrary.h"
#include "Kismet/GameplayStatics.h"
#include "Widgets/Layout/SScaleBox.h"
#include "Styling/CoreStyle.h"
#include "StoveSDKObject.h"

#define LOCTEXT_NAMESPACE "StoveSDKPopup"

#define CHECK_HTTP_REQUEST_VALIDITY(res) \
    if (!res.IsValid() || !EHttpResponseCodes::IsOk(res->GetResponseCode())) \
    { \
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::GetButtonTextureFromURL - HTTP request failed, status code %d"), res->GetResponseCode()); \
        SetCloseButtonAsDefault(); \
        return; \
    } \

#define CHECK_TEXTURE_VALIDITY(texturePtr) \
    if (!texturePtr.IsValid()) \
    { \
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::GetButtonTextureFromURL - Invalid texture")); \
        SetCloseButtonAsDefault(); \
        return; \
    } \

#define AS_TATTRIBUTE_BOOL(func) TAttribute<bool>::Create( \
                                    TAttribute<bool>::FGetter::CreateLambda( \
                                        [this]() -> const bool { return func(); } \
                                    ) \
                                ) \

UStoveSDKPopup::UStoveSDKPopup(const FObjectInitializer &ObjectInitializer)
    : Super(ObjectInitializer)
    , _NaviButtonSize(28.0f)
    , _MarginDefaultSize(152)
    , _DisallowFontSize(18)
    , _PopupIndex(0)
    , _CloseButtonPresetCount(6)
    , _DefaultCloseButtonStyleType(1)
    , _bUseDimmedBackground(true)
    , _bIsNotFrame{ false }
    , _ButtonPressedPadding(FMargin(10, 10, 0, 0))
    , _VertialAlign{ VAlign_Center }
    , _HorizontalAlign{ HAlign_Center }
    , _FrameWidth(800)
{	
    // Close Button Brush
    for (int i = 0; i < _CloseButtonPresetCount; i++)
    {
        FButtonStyle *CloseButtonStyle = new FButtonStyle();
        FSlateDynamicImageBrush *CloseButtonBrushNormal = new FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                *FString::Format(
                    TEXT("Texture2D'/StoveSDKPlugin/Textures/close_btn_sample_{0}.close_btn_sample_{0}'"),
                    TArray<FStringFormatArg>({ FStringFormatArg(i + 1) })
                )
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("closeButtonBrushNormal")
        );

        FSlateDynamicImageBrush *CloseButtonBrushPressed = new FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                *FString::Format(
                    TEXT("Texture2D'/StoveSDKPlugin/Textures/close_btn_sample_{0}_pressed.close_btn_sample_{0}_pressed'"),
                    TArray<FStringFormatArg>({ FStringFormatArg(i + 1) })
                )
                ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("closeButtonBrushPressed")
        );

        CloseButtonStyle->SetNormal(*CloseButtonBrushNormal);
        CloseButtonStyle->SetHovered(*CloseButtonBrushNormal);
        CloseButtonStyle->SetPressed(*CloseButtonBrushPressed);
        CloseButtonStyle->SetPressedPadding(_ButtonPressedPadding);
        CloseButtonStyle->SetDisabled(*CloseButtonBrushNormal);
        _CloseButtonStyles.Push(CloseButtonStyle);
    }

    // Go Back Button Brush
    _GoBackButtonStyle = new FButtonStyle();
    FSlateDynamicImageBrush *GoBackActive = new FSlateDynamicImageBrush(
        ConstructorHelpers::FObjectFinder<UTexture2D>(
            TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopBackWhiteActive_3x.btnPopBackWhiteActive_3x'")
        ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("GoBackActive")
    );
    _GoBackButtonStyle->SetNormal(*GoBackActive);
    _GoBackButtonStyle->SetHovered(*GoBackActive);
    _GoBackButtonStyle->SetPressed(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopBackWhitePressed_3x.btnPopBackWhitePressed_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("GoBackPressed")
        )
    );
    _GoBackButtonStyle->SetPressedPadding(_ButtonPressedPadding);
    _GoBackButtonStyle->SetDisabled(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopBackWhiteInactive_3x.btnPopBackWhiteInactive_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("GoBackDisabled")
        )
    );

    // Go Forward Button Brush
    _GoForwardButtonStyle = new FButtonStyle();
    FSlateDynamicImageBrush *GoForwardActive = new FSlateDynamicImageBrush(
        ConstructorHelpers::FObjectFinder<UTexture2D>(
            TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopForwardWhiteActive_3x.btnPopForwardWhiteActive_3x'")
        ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("GoForwardActive")
    );
    _GoForwardButtonStyle->SetNormal(*GoForwardActive);
    _GoForwardButtonStyle->SetHovered(*GoForwardActive);
    _GoForwardButtonStyle->SetPressed(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopForwardWhitePressed_3x.btnPopForwardWhitePressed_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("GoForwardPressed")
        )
    );
    _GoForwardButtonStyle->SetPressedPadding(_ButtonPressedPadding);
    _GoForwardButtonStyle->SetDisabled(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopForwardWhiteInactive_3x.btnPopForwardWhiteInactive_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("GoForwardDisabled")
        )
    );

    // Reload Button Brush
    _ReloadButtonStyle = new FButtonStyle();
    FSlateDynamicImageBrush *ReloadActive = new FSlateDynamicImageBrush(
        ConstructorHelpers::FObjectFinder<UTexture2D>(
            TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopRefreshWhiteActive_3x.btnPopRefreshWhiteActive_3x'")
        ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("reloadActive")
    );
    _ReloadButtonStyle->SetNormal(*ReloadActive);
    _ReloadButtonStyle->SetHovered(*ReloadActive);
    _ReloadButtonStyle->SetPressed(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopRefreshWhitePressed_3x.btnPopRefreshWhitePressed_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("reloadPressed")
        )
    );
    _ReloadButtonStyle->SetPressedPadding(_ButtonPressedPadding);
    _ReloadButtonStyle->SetDisabled(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopRefreshWhiteInactive_3x.btnPopRefreshWhiteInactive_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("reloadDisabled")
        )
    );

    // Home Button Brush
    _HomeButtonStyle = new FButtonStyle();
    FSlateDynamicImageBrush *HomeActive = new FSlateDynamicImageBrush(
        ConstructorHelpers::FObjectFinder<UTexture2D>(
            TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopHomeWhiteActive_3x.btnPopHomeWhiteActive_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("homeActive")
    );
    _HomeButtonStyle->SetNormal(*HomeActive);
    _HomeButtonStyle->SetHovered(*HomeActive);
    _HomeButtonStyle->SetPressed(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopHomeWhitePressed_3x.btnPopHomeWhitePressed_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("homePressed")
        )
    );
    _HomeButtonStyle->SetPressedPadding(_ButtonPressedPadding);
    _HomeButtonStyle->SetDisabled(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopHomeWhiteInactive_3x.btnPopHomeWhiteInactive_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("homeDisabled")
        )
    );

    // Disallowed Button Brush
    _DisallowedButtonStyle = new FButtonStyle();
    FSlateDynamicImageBrush *DisallowedActive = new FSlateDynamicImageBrush(
        ConstructorHelpers::FObjectFinder<UTexture2D>(
            TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopDisallowWhiteActive_3x.btnPopDisallowWhiteActive_3x'")
            ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("disallowedActive")
    );
    _DisallowedButtonStyle->SetNormal(*DisallowedActive);
    _DisallowedButtonStyle->SetHovered(*DisallowedActive);
    _DisallowedButtonStyle->SetPressed(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopDisallowWhitePressed_3x.btnPopDisallowWhitePressed_3x'")
                ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("disallowedPressed")
        )
    );
    _DisallowedButtonStyle->SetPressedPadding(_ButtonPressedPadding);
    _DisallowedButtonStyle->SetDisabled(
        FSlateDynamicImageBrush(
            ConstructorHelpers::FObjectFinder<UTexture2D>(
                TEXT("Texture2D'/StoveSDKPlugin/Textures/btnPopDisallowWhiteInactive_3x.btnPopDisallowWhiteInactive_3x'")
                ).Object, FVector2D(_NaviButtonSize, _NaviButtonSize), FName("disallowedDisabled")
        )
    );

    _ImageWrapperModule = &FModuleManager::LoadModuleChecked<IImageWrapperModule>(TEXT("ImageWrapper"));
}

void UStoveSDKPopup::OnClickClose()
{
    _PopupIndex++;
    bool bHasNextPopup = { _PopupIndex < _PopupList.Num() };
    if (bHasNextPopup)
    {
        SetNewWebview();
        LoadURL();
        SetDisallowedText();
        SetPopupItemsVisibility();
        SetCloseButtonStyle();
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::OnClickClose - current index = %d, popup count = %d"), _PopupIndex, _PopupList.Num());
    }
    else
    {
        UWidgetBlueprintLibrary::SetInputMode_GameAndUIEx(UGameplayStatics::GetPlayerController(GetWorld(), 0));
        _Webview.Reset();
        if (_WebInterface.IsValid())
        {
            _WebInterface->Destroy();
            _WebInterface.Reset();
        }
        this->RemoveFromParent();
        _OnClose();
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::OnClickClose - close webview"));
    }
}

void UStoveSDKPopup::OnClickGoBack()
{
    if (_WebInterface.IsValid())
    {
        _WebInterface->GoBack();
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::OnClickGoBack"));
    }
}

void UStoveSDKPopup::OnClickGoForward()
{
    if (_WebInterface.IsValid())
    {
        _WebInterface->GoForward();
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::OnClickGoForward"));
    }
}

void UStoveSDKPopup::OnClickReload()
{
    if (_WebInterface.IsValid())
    {
        _WebInterface->Reload();
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::OnClickReload"));
    }
}

void UStoveSDKPopup::OnClickGoHome()
{
    if (_WebInterface.IsValid())
    {
        _WebInterface->GoHome();
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::OnClickHome"));
    }
}

void UStoveSDKPopup::OnClickDisallowed()
{
    if (_WebInterface.IsValid())
    {
        // TODO: implement function
        FindStoveSDKObject()->StoveSDKSetPopupDisallowed(_PopupList[0].Id, _PopupList[0].Control.UI.DisallowedDay);
        OnClickClose();
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::OnClickDisallowed"));
    }
}

void UStoveSDKPopup::RegisterCloseCallback(const CloseCallback& closeCallback)
{
    _OnClose = closeCallback;
}

void UStoveSDKPopup::SetAlignment(const EStoveAlignment& StoveAlignment)
{
    switch (StoveAlignment)
    {
    case EStoveAlignment::TopLeft:
        _VertialAlign = VAlign_Top;
        _HorizontalAlign = HAlign_Left;
        break;
    case EStoveAlignment::TopCenter:
        _VertialAlign = VAlign_Top;
        _HorizontalAlign = HAlign_Center;
        break;
    case EStoveAlignment::TopRight:
        _VertialAlign = VAlign_Top;
        _HorizontalAlign = HAlign_Right;
        break;
    case EStoveAlignment::MiddleLeft:
        _VertialAlign = VAlign_Center;
        _HorizontalAlign = HAlign_Left;
        break;
    case EStoveAlignment::MiddleCenter:
        _VertialAlign = VAlign_Center;
        _HorizontalAlign = HAlign_Center;
        break;
    case EStoveAlignment::MiddleRight:
        _VertialAlign = VAlign_Center;
        _HorizontalAlign = HAlign_Right;
        break;
    case EStoveAlignment::BottomLeft:
        _VertialAlign = VAlign_Bottom;
        _HorizontalAlign = HAlign_Left;
        break;
    case EStoveAlignment::BottomCenter:
        _VertialAlign = VAlign_Bottom;
        _HorizontalAlign = HAlign_Center;
        break;
    case EStoveAlignment::BottomRight:
        _VertialAlign = VAlign_Bottom;
        _HorizontalAlign = HAlign_Right;
        break;
    default:
        _VertialAlign = VAlign_Center;
        _HorizontalAlign = HAlign_Center;
        break;
    }
}

void UStoveSDKPopup::SetNoFrame(const bool& NoFrame)
{
    _bIsNotFrame = NoFrame;
}

void UStoveSDKPopup::Initiate(TSharedPtr<IStoveSDKWebInterface> NewWebInterface)
{
    _WebInterface = NewWebInterface;
    if (_WebInterface.IsValid())
    {
        _WebInterface->RegisterNotificationListner([this](const FString& Name, const FString& Value, const FString& Id) {

            if (Name == "Close" && Value == "popup" && Id == "1")
            {
                OnClickClose();
            }
            
        });
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::SetWebView - Success"));
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::SetWebView - Invalid webview"));
    }
}

void UStoveSDKPopup::SetPopupList(const TArray<FPopupUIFrame> &Popups)
{
    _PopupList = Popups;
    _PopupIndex = 0;
}

void UStoveSDKPopup::EnableDimmedBackground(const bool EnableDimmedBackground)
{
    _bUseDimmedBackground = EnableDimmedBackground;
}

void UStoveSDKPopup::Display()
{
    if (_PopupList.Num() == 0)
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::Display - Popup list is empty"));
        return;
    }

    this->AddToViewport(10);
    UWidgetBlueprintLibrary::SetInputMode_UIOnlyEx(UGameplayStatics::GetPlayerController(GetWorld(), 0), this);
    LoadURL();

    UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::Display"));
}

void UStoveSDKPopup::LoadURL()
{
    if (_WebInterface.IsValid())
    {

        if (_Cookies.Num() > 0)
        {
            for (auto Cookie : _Cookies)
            {
                _WebInterface->SetCookie(_PopupList[_PopupIndex].Url, Cookie);
            }
            
        }

        TArray<FStoveCefHeader> StoveHeaders;
        for (auto Header : _Headers)
        {
            StoveHeaders.Add(FStoveCefHeader{ Header.Name, Header.Value });
        }
        _WebInterface->LoadURL(_PopupList[_PopupIndex].Url, StoveHeaders);
    }
}

void UStoveSDKPopup::SetHeaders(const TArray<FStovePopupRequestHeader> &Headers)
{
    _Headers = Headers;
}

void UStoveSDKPopup::SetCookies(const TArray<FStoveCefCookie> &Cookies)
{
    _Cookies = Cookies;
}

TSharedRef<SWidget> UStoveSDKPopup::RebuildWidget()
{
    if (IsDesignTime())
    {
        return SNew(SBox)
            .HAlign(HAlign_Center)
            .VAlign(VAlign_Center)
            [
                SNew(STextBlock)
                .Text(LOCTEXT("StoveSDKPopup", "StoveSDKPopup"))
            ];
    }
    else
    {
        if (!_Frame.IsValid())
        {
            if (_bIsNotFrame)
            {
                 SAssignNew(_Frame, SOverlay)
//                 + SOverlay::Slot()
//                 [
//                     SAssignNew(dimmedBackground, SBox)
//                     .WidthOverride(viewportSize.X)
//                     .HeightOverride(viewportSize.Y)
//                     [
//                         SNew(SBorder)
//                         .BorderImage(new FSlateColorBrush(FLinearColor(FColor(0, 0, 0, 100))))
//                     ]
//                 ]
                 + SOverlay::Slot()
                 //.Padding(100.0f, 200.0f, 300.0f, 400.0f)
                 .HAlign(_HorizontalAlign)
                 .VAlign(_VertialAlign)
                 [
                      SNew(SScaleBox)
                      .HAlign(HAlign_Fill)
                      .VAlign(VAlign_Fill)
                      .Stretch(EStretch::ScaleToFill)
                      //.UserSpecifiedScale(1.0f)
                      //.IgnoreInheritedScale(true)
                      [
                         SNew(SBox)
                         .WidthOverride(420)
                         .HeightOverride(700)
                         //.MaxAspectRatio(1.0f)
                         //.Padding(FMargin(marginLeft, marginY, marginRight, marginY))
                         [
                            SAssignNew(_WebviewHolder, SBorder)
                           .BorderImage(new FSlateColorBrush(FLinearColor(FColor(255, 255, 255))))
                         ]
                     ]
                ];
            }
            else
            {
                FIntPoint ViewportSize = GEngine->GameViewport->Viewport->GetSizeXY();

                FIntPoint PopupSize;        // only PopupWidth
                PopupSize.X = ViewportSize.X - _MarginDefaultSize * 2;
                PopupSize.Y = (PopupSize.X / 16) * 9;

                FIntPoint FrameSize;    // PopupWidth + CloseSpacer + CloseButton
                FrameSize.X = PopupSize.X + 8 + 40;
                FrameSize.Y = PopupSize.Y;

                FIntPoint NavigationBarSize;
                NavigationBarSize.X = PopupSize.X;
                NavigationBarSize.Y = 52;

                float FrameWidthRatio =  float(FrameSize.X) / float(ViewportSize.X);
                float LeftMarginRatio = float(_MarginDefaultSize) / float(ViewportSize.X);
                float RightMarginRatio = 1.0f - (FrameWidthRatio + LeftMarginRatio);
                float TotalWidthVerify = FrameWidthRatio + LeftMarginRatio + RightMarginRatio; // 1.0f verify

                float NavigationBarHeightRatio = 52.0f / float(ViewportSize.Y);
                float FrameHeightRatio = (float(FrameSize.Y) / float(ViewportSize.Y));
                float FrameTopHeightRatio = (1.0f - FrameHeightRatio) / 2;
                float FrameBottomHeightRatio = FrameTopHeightRatio;
                float TotalHeightVerify = FrameHeightRatio + FrameTopHeightRatio + FrameBottomHeightRatio; // 1.0f verify

                float PopupWidthRatio = float(PopupSize.X) / float(FrameSize.X);
                float CloseSpacerWidthRatio = 8.0f / float(FrameSize.X);
                float CloseButtionWidthRatio = 40.0f / float(FrameSize.X);
                float TotalPopupWidthVerify = PopupWidthRatio + CloseSpacerWidthRatio + CloseButtionWidthRatio; // 1.0f verify

                float NaviTotalSpacerWidthRatio = 28.0f / float(FrameSize.X);
                float NaviButtonWidthRatio = NaviTotalSpacerWidthRatio;
                float NaviButtonSpacerWidthRatio = 12.0f / float(FrameSize.X);
                float NaviTextSpacerWidthRatio = 4.0f / float(FrameSize.X);
                float NaviCenterSpacerWidthRatio = 1.0f - (NaviTotalSpacerWidthRatio*2
                                                           + NaviTextSpacerWidthRatio
                                                           + NaviButtonWidthRatio*5);

                float NaviTobBottomMarginHeightRatio = 12.0f / float(NavigationBarSize.Y);
                float NaviButtonSizeHeightRatio = 1.0f - (NaviTobBottomMarginHeightRatio * 2);

                float NaviOverlayContentsHeightRatio = 52.0f/ float(PopupSize.Y);
                float NaviOverlayTopMarginRatio = 1.0f - NaviOverlayContentsHeightRatio;

                UE_LOG(LogStoveSDKPlugin, Log, TEXT("Viewport x: %d, y: %d"), ViewportSize.X, ViewportSize.Y);
                UE_LOG(LogStoveSDKPlugin, Log, TEXT("Frame    x: %d, y: %d"), PopupSize.X, PopupSize.Y);
                //UE_LOG(LogStoveSDKPlugin, Log, TEXT("LeftMargin %d"), marginLeft);

                SAssignNew(_Frame, SOverlay)
                + SOverlay::Slot()
#pragma region DimdBackground
                [
                    SAssignNew(_DimmedBackground, SBox)
                    .WidthOverride(ViewportSize.X)
                    .HeightOverride(ViewportSize.Y)
                    [
                        SNew(SBorder)
                        .BorderImage(new FSlateColorBrush(FLinearColor(FColor(31, 31, 31, 179))))
                    ]
                ]
#pragma endregion DimdBackground
                + SOverlay::Slot()
#pragma region PopupPanel
                .HAlign(HAlign_Fill)
                .VAlign(VAlign_Fill)
                .Padding(FMargin(0, 0, 0, 0))
                [
                    SNew(SHorizontalBox)    //Frame 전체
                    + SHorizontalBox::Slot()
#pragma region LeftMargin
                    .FillWidth(LeftMarginRatio)
                    [
                        SNew(SBox)
                        [
                            SNew(SSpacer)
                        ]
                    ]
#pragma endregion LeftMargin
                    + SHorizontalBox::Slot()
#pragma region Frame
                    .FillWidth(FrameWidthRatio)
                    [
                        SNew(SVerticalBox)
                        + SVerticalBox::Slot()
#pragma region Frame_Top_Margin
                        .FillHeight(FrameTopHeightRatio)
                        [
                            SNew(SSpacer)
                        ]
#pragma endregion Frame_Top_Margin
                        + SVerticalBox::Slot()
#pragma region Frame_Contents
                        .FillHeight(FrameHeightRatio)
                        [
                            SNew(SOverlay)
                            + SOverlay::Slot()
#pragma region CloseOverlay
                            .VAlign(VAlign_Top)
                            [
                                SNew(SHorizontalBox)
                                + SHorizontalBox::Slot()
                                .FillWidth(PopupWidthRatio)
                                [
                                   SNew(SSpacer)
                                ]
                                + SHorizontalBox::Slot()
                                .VAlign(VAlign_Top)
                                .FillWidth(CloseSpacerWidthRatio)
                                [
                                    SNew(SScaleBox)
                                    .IgnoreInheritedScale(true)
                                    [
                                        SNew(SBox)
                                        .WidthOverride(8)
                                        .MaxDesiredWidth(8)
                                        [
                                            SNew(SSpacer)
                                        ]
                                    ]
                                 ]
                                + SHorizontalBox::Slot()
                                .FillWidth(CloseButtionWidthRatio)
                                [
                                    SNew(SScaleBox)
                                    .IgnoreInheritedScale(true)
                                    [
                                        SAssignNew(_CloseBox, SBox)
                                        .WidthOverride(40)
                                        .HeightOverride(40)
                                        .MaxDesiredHeight(40)
                                        [
                                            SAssignNew(_CloseButton, SButton)
                                            .ButtonStyle(new FButtonStyle())
                                            .OnClicked(FOnClicked::CreateLambda([this] {
                                                OnClickClose();
                                                return FReply::Handled();
                                            }))
                                        ]
                                    ]
                                ]
                            ]
#pragma endregion CloseOverlay
                            + SOverlay::Slot()
#pragma region WebOverlay
                            [
                                SNew(SHorizontalBox)
                                + SHorizontalBox::Slot()
                                .FillWidth(PopupWidthRatio)
                                [
                                    SNew(SBox)
                                    [
                                        SAssignNew(_WebviewHolder, SBorder)
                                        .BorderImage(new FSlateColorBrush(FLinearColor(FColor(255, 255, 255))))
                                    ]
                                ]
                                + SHorizontalBox::Slot()
                                .FillWidth(CloseSpacerWidthRatio+CloseButtionWidthRatio)
                                [
                                    SNew(SSpacer)
                                ]
                            ]
#pragma endregion WebOverlay
                            + SOverlay::Slot()
#pragma region NaviOverlay
                            [
                                SAssignNew(_NavigationBar, SVerticalBox)
                                + SVerticalBox::Slot()
                                .FillHeight(NaviOverlayTopMarginRatio)
                                + SVerticalBox::Slot()
                                .FillHeight(NaviOverlayContentsHeightRatio)
                                [
                                    SNew(SHorizontalBox)
                                    + SHorizontalBox::Slot()
                                    .FillWidth(PopupWidthRatio)
                                    [
                                        SNew(SBorder)
                                        .BorderImage(new FSlateColorBrush(FLinearColor(FColor(31, 31, 31))))
                                        [
                                            SNew(SVerticalBox)
                                            + SVerticalBox::Slot()
                                            .FillHeight(NaviTobBottomMarginHeightRatio)
                                            + SVerticalBox::Slot()
#pragma region NaviButtons
                                            .FillHeight(NaviButtonSizeHeightRatio)
                                            [
                                                SNew(SHorizontalBox)
                                                + SHorizontalBox::Slot()
                                                .FillWidth(NaviTotalSpacerWidthRatio)
                                                + SHorizontalBox::Slot()
#pragma region GoBack
                                                .FillWidth(NaviButtonWidthRatio)
                                                [
                                                    SAssignNew(_GoBackBox, SBox)
                                                    [
                                                        SNew(SButton)
                                                        .ButtonStyle(_GoBackButtonStyle)
                                                        .IsEnabled(AS_TATTRIBUTE_BOOL(IsGoBackEnabled))
                                                        .OnClicked(FOnClicked::CreateLambda([this] {
                                                            OnClickGoBack();
                                                            return FReply::Handled();
                                                        }))
                                                    ]
                                                ]
#pragma endregion GoBack
                                                + SHorizontalBox::Slot()
                                                .FillWidth(NaviButtonSpacerWidthRatio)
                                                + SHorizontalBox::Slot()
#pragma region GoFowrd
                                                .FillWidth(NaviButtonWidthRatio)
                                                [
                                                    SAssignNew(_GoForwardBox, SBox)
                                                    [
                                                        SNew(SButton)
                                                        .ButtonStyle(_GoForwardButtonStyle)
                                                        .IsEnabled(AS_TATTRIBUTE_BOOL(IsGoForwardEnabled))
                                                        .OnClicked(FOnClicked::CreateLambda([this] {
                                                            OnClickGoForward();
                                                            return FReply::Handled();
                                                        }))
                                                    ]
                                                ]
#pragma endregion GoFowrd
                                                + SHorizontalBox::Slot()
                                                .FillWidth(NaviButtonSpacerWidthRatio)
                                                + SHorizontalBox::Slot()
#pragma region ReLoad
                                                .FillWidth(NaviButtonWidthRatio)
                                                [
                                                    SAssignNew(_ReloadBox, SBox)
                                                    [
                                                        SNew(SButton)
                                                        .ButtonStyle(_ReloadButtonStyle)
                                                        .IsEnabled(AS_TATTRIBUTE_BOOL(IsReloadEnabled))
                                                        .OnClicked(FOnClicked::CreateLambda([this] {
                                                            OnClickReload();
                                                            return FReply::Handled();
                                                        }))
                                                    ]
                                                ]
#pragma endregion ReLoad
                                                + SHorizontalBox::Slot()
                                                .FillWidth(NaviButtonSpacerWidthRatio)
                                                + SHorizontalBox::Slot()
#pragma region GoHome
                                                .FillWidth(NaviButtonWidthRatio)
                                                [
                                                    SAssignNew(_HomeBox, SBox)
                                                    [
                                                        SNew(SButton)
                                                        .ButtonStyle(_HomeButtonStyle)
                                                        .IsEnabled(AS_TATTRIBUTE_BOOL(IsHomeEnabled))
                                                        .OnClicked(FOnClicked::CreateLambda([this] {
                                                            OnClickGoHome();
                                                            return FReply::Handled();
                                                        }))
                                                    ]
                                                ]
#pragma endregion GoHome
                                                + SHorizontalBox::Slot()
#pragma region DisallowText
                                .FillWidth(NaviCenterSpacerWidthRatio)
                                .HAlign(HAlign_Right)
                                .VAlign(VAlign_Center)
                                [
                                    SAssignNew(_DisallowedText, STextBlock)
                                    .Font(FSlateFontInfo(FCoreStyle::GetDefaultFont(), _DisallowFontSize))
                                ]
#pragma endregion DisallowText
                                                + SHorizontalBox::Slot()
                                                .FillWidth(NaviTextSpacerWidthRatio)
                                                + SHorizontalBox::Slot()
#pragma region DisallowButton
                                                .FillWidth(NaviButtonWidthRatio)
                                                [
                                                    SAssignNew(_DisallowedBox, SBox)
                                                    [
                                                        SNew(SButton)
                                                        .ButtonStyle(_DisallowedButtonStyle)
                                                        .OnClicked(FOnClicked::CreateLambda([this] {
                                                            OnClickDisallowed();
                                                            return FReply::Handled();
                                                        }))
                                                    ]
                                                ]
#pragma endregion DisallowButton
                                                + SHorizontalBox::Slot()
                                                .FillWidth(NaviTotalSpacerWidthRatio)
                                            ]
#pragma endregion NaviButtons
                                            + SVerticalBox::Slot()
                                            .FillHeight(NaviTobBottomMarginHeightRatio)
                                        ]
                                    ]
                                    + SHorizontalBox::Slot()
                                    .FillWidth(CloseButtionWidthRatio + CloseSpacerWidthRatio)
                                ]
                            ]
#pragma endregion NaviOverlay
                        ]
#pragma endregion Frame_Contents
                        + SVerticalBox::Slot()
#pragma region Frame_Bottom_Margin
                        .FillHeight(FrameBottomHeightRatio)
                        [
                            SNew(SSpacer)
                        ]
#pragma endregion Frame_Bottom_Margin
                    ]
#pragma endregion Frame
                    + SHorizontalBox::Slot()
#pragma region RightMargin
                        .FillWidth(RightMarginRatio)
                        [
                            SNew(SBox)
                            [
                                SNew(SSpacer)
                            ]
                        ]
#pragma endregion RightMargin
                ];
#pragma endregion PopupPanel
            }
        }

        SetDimmedBackground();
        SetNewWebview();
        SetDisallowedText();
        SetPopupItemsVisibility();
        SetCloseButtonStyle();
        return _Frame.ToSharedRef();
    }
}

bool UStoveSDKPopup::IsGoBackEnabled() const
{
    if (_WebInterface.IsValid())
    {
        return _WebInterface->CanGoBack();
    }
    return false;
}

bool UStoveSDKPopup::IsGoForwardEnabled() const
{
    if (_WebInterface.IsValid())
    {
        return _WebInterface->CanGoForward();
    }
    return false;
}

bool UStoveSDKPopup::IsReloadEnabled() const
{
    // always can reload page
    return true;
}

bool UStoveSDKPopup::IsHomeEnabled() const
{
    if (_WebInterface.IsValid())
    {
        return _WebInterface->CanGoBack();
    }
    return false;
}

void UStoveSDKPopup::SetDimmedBackground()
{
    if (_DimmedBackground.IsValid())
    {
        _DimmedBackground->SetVisibility(_bUseDimmedBackground ? EVisibility::Visible : EVisibility::Collapsed);
    }
}

void UStoveSDKPopup::SetNewWebview()
{
    if (!_WebviewHolder.IsValid())
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::SetNewWebview - Invalid webview holder"));
        return;
    }

    _Webview.Reset();
    if (_WebInterface.IsValid())
    {
        _WebInterface->Create(_Webview);
    }

    _WebviewHolder->SetContent(
        SNew(SOverlay)
        + SOverlay::Slot()
        .HAlign(HAlign_Center)
        .VAlign(VAlign_Center)
        [
            SNew(SCircularThrobber)
            .Radius(_NaviButtonSize)
            /*.Visibility(TAttribute<EVisibility>::Create(TAttribute<EVisibility>::FGetter::CreateLambda(
                [this]()
                {
                    return currentPopupControl.ui.useLoadingAnimation ? EVisibility::Visible : EVisibility::Hidden;
                }
            )))*/
            // TODO: set visibility dynamically depening on initialization status of webview
            .Visibility(_PopupList[_PopupIndex].bUseLoadingAnimation ? EVisibility::Visible : EVisibility::Hidden)
        ]
        + SOverlay::Slot()
        [
            SNew(SWeakWidget).PossiblyNullContent(
                _Webview.IsValid() ? _Webview : SNew(SBox)
            )
        ]
    );
}

void UStoveSDKPopup::SetDisallowedText()
{
    if (_DisallowedText.IsValid())
    {
        if (_PopupList[_PopupIndex].Days == 1)
        {
            FString DisallowedString = FindStoveSDKObject()->StoveSDKTranslateLanguage("stove_view_do_not_show_today");
            _DisallowedText->SetText(FText::FromString(DisallowedString));
        }
        else
        {
            FString DisallowedString = FindStoveSDKObject()->StoveSDKTranslateLanguage("stove_view_do_not_show_week");
            _DisallowedText->SetText(FText::FromString(DisallowedString));
        }
        
    }
}

void UStoveSDKPopup::SetPopupItemsVisibility()
{
    if (_CloseBox.IsValid())
    {
        _CloseBox->SetVisibility(_PopupList[_PopupIndex].bCloseButton ? EVisibility::Visible : EVisibility::Hidden);
    }

    if (_NavigationBar.IsValid())
    {
        if (_PopupList[_PopupIndex].bNavigationBar)
        {
            // _NavigationBar->SetVisibility(EVisibility::HitTestInvisible);
            if (_GoBackBox.IsValid())
            {
                _GoBackBox->SetVisibility(_PopupList[_PopupIndex].bBackButton ? EVisibility::Visible : EVisibility::Hidden);
            }
            if (_GoForwardBox.IsValid())
            {
                _GoForwardBox->SetVisibility(_PopupList[_PopupIndex].bForwardButton ? EVisibility::Visible : EVisibility::Hidden);
            }
            if (_ReloadBox.IsValid())
            {
                _ReloadBox->SetVisibility(_PopupList[_PopupIndex].bRefreshButton ? EVisibility::Visible : EVisibility::Hidden);
            }
            if (_HomeBox.IsValid())
            {
                _HomeBox->SetVisibility(_PopupList[_PopupIndex].bHomeButton ? EVisibility::Visible : EVisibility::Hidden);
            }
            if (_DisallowedBox.IsValid())
            {
                _DisallowedBox->SetVisibility(_PopupList[_PopupIndex].bDisallowedButton ? EVisibility::Visible : EVisibility::Hidden);
                _DisallowedText->SetVisibility(_PopupList[_PopupIndex].bDisallowedButton ? EVisibility::Visible : EVisibility::Hidden);
            }
        }
        else
        {
            _NavigationBar->SetVisibility(EVisibility::Hidden);
        }
    }
}

void UStoveSDKPopup::SetCloseButtonStyle()
{
    if (!_CloseButton.IsValid())
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::SetCloseButtonStyle - closeButton is invalid"));
        return;
    }

    int CloseButtonType = _PopupList[_PopupIndex].CloseType;
    auto DefaultButtonUrl = _PopupList[_PopupIndex].CloseNormalUrl;
    auto PressedButtonUrl = _PopupList[_PopupIndex].ClosePressedUrl;
    if (DefaultButtonUrl.IsEmpty() || PressedButtonUrl.IsEmpty())
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("UStoveSDKPopup::SetCloseButtonStyle - ButtonUrl is Empty()"));
    }
    if (CloseButtonType == 0 && !DefaultButtonUrl.IsEmpty() && !PressedButtonUrl.IsEmpty())
    {
        GetButtonTextureFromURL(DefaultButtonUrl, PressedButtonUrl);
    }
    else
    {
        int FinalButtonType = CloseButtonType >= 1 && CloseButtonType <= 6 ? CloseButtonType - 1 : _DefaultCloseButtonStyleType - 1;
        _CloseButton->SetButtonStyle(_CloseButtonStyles[FinalButtonType]);
    }
}

void UStoveSDKPopup::GetButtonTextureFromURL(FString &DefaultButtonURL, FString &PressedButtonURL)
{

    auto DefaultButtonRequest = FHttpModule::Get().CreateRequest();

    DefaultButtonRequest->SetVerb("GET");
    DefaultButtonRequest->SetURL(DefaultButtonURL);
    DefaultButtonRequest->OnProcessRequestComplete().BindLambda(
        [this, DefaultButtonURL, PressedButtonURL](FHttpRequestPtr DefaultButtonRequest, FHttpResponsePtr DefaultButtonResponse, bool DefaultButtonSuccess)
        {
            CHECK_HTTP_REQUEST_VALIDITY(DefaultButtonResponse);

            auto PressedButtonRequest = FHttpModule::Get().CreateRequest();
            PressedButtonRequest->SetURL(PressedButtonURL);
            PressedButtonRequest->OnProcessRequestComplete().BindLambda(
                [this, DefaultButtonResponse](FHttpRequestPtr PressedButtonRequest, FHttpResponsePtr PressedButtonResponse, bool PressedButtonSuccess)
                {
                    CHECK_HTTP_REQUEST_VALIDITY(PressedButtonResponse);

#if VERSION_GREATER_OR_EQUAL_423
                    _CloseButtonTextureDefaultFromWeb = Async<>(
#else
                    _CloseButtonTextureDefaultFromWeb = Async<UTexture2D*>(
#endif
                        EAsyncExecution::ThreadPool,
                        [=]() -> UTexture2D*
                        {
                            return MakeTexture(DefaultButtonResponse->GetContent());
                        },
                        [=]()
                        {
                            CHECK_TEXTURE_VALIDITY(_CloseButtonTextureDefaultFromWeb);

#if VERSION_GREATER_OR_EQUAL_423
                            _CloseButtonTexturePressedFromWeb = Async<>(
#else
                            _CloseButtonTexturePressedFromWeb = Async<UTexture2D*>(
#endif
                                EAsyncExecution::ThreadPool,
                                [=]() -> UTexture2D*
                                {
                                    return MakeTexture(PressedButtonResponse->GetContent());
                                },
                                [=]()
                                {
                                    CHECK_TEXTURE_VALIDITY(_CloseButtonTexturePressedFromWeb);

                                    FButtonStyle *CloseButtonStyle = new FButtonStyle();
                                    const FSlateBrush *CloseBrushDefaultFromWeb = FDeferredCleanupSlateBrush::CreateBrush(
                                        FSlateImageBrush(_CloseButtonTextureDefaultFromWeb.Get(), FVector2D(_NaviButtonSize, _NaviButtonSize))
                                    ).Get().GetSlateBrush();
                                    const FSlateBrush *CloseBrushPressedFromWeb = FDeferredCleanupSlateBrush::CreateBrush(
                                        FSlateImageBrush(_CloseButtonTexturePressedFromWeb.Get(), FVector2D(_NaviButtonSize, _NaviButtonSize))
                                    ).Get().GetSlateBrush();

                                    CloseButtonStyle->SetNormal(*CloseBrushDefaultFromWeb);
                                    CloseButtonStyle->SetHovered(*CloseBrushPressedFromWeb);
                                    CloseButtonStyle->SetPressed(*CloseBrushPressedFromWeb);
                                    CloseButtonStyle->SetPressedPadding(_ButtonPressedPadding);
                                    CloseButtonStyle->SetDisabled(*CloseBrushDefaultFromWeb);

                                    _CloseButton->SetButtonStyle(CloseButtonStyle);
                                    UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::GetButtonTextureFromURL - Success"));
                                }
                            );
                        }
                    );

                }
            );
            PressedButtonRequest->ProcessRequest();
        }
    );
    DefaultButtonRequest->ProcessRequest();
}

UTexture2D* UStoveSDKPopup::MakeTexture(const TArray<uint8> &Content)
{
    TSharedPtr<IImageWrapper> ImageWrapper = _ImageWrapperModule->CreateImageWrapper(
        _ImageWrapperModule->DetectImageFormat(Content.GetData(), Content.Num())
    );
    if (!ImageWrapper.IsValid())
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::GetButtonTextureFromURL - Failed to create image wrapper"));
        return nullptr;
    }

#if VERSION_GREATER_OR_EQUAL_425
    TArray<uint8> RawData;
    ImageWrapper->SetCompressed(Content.GetData(), Content.Num());
    ImageWrapper->GetRaw(ERGBFormat::BGRA, 8, RawData);
    if (RawData.Num() <= 0)
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::GetButtonTextureFromURL - Failed to decompress image file"));
        return nullptr;
    }

    UTexture2D *ButtonTexture = UTexture2D::CreateTransient(ImageWrapper->GetWidth(), ImageWrapper->GetHeight());

#if ENGINE_MAJOR_VERSION == 5
    FTexture2DMipMap* Mips = &ButtonTexture->GetPlatformData()->Mips[0];
#else
    FTexture2DMipMap* Mips = &ButtonTexture->PlatformData->Mips[0];
#endif

    FMemory::Memcpy(Mips->BulkData.Lock(LOCK_READ_WRITE), RawData.GetData(), RawData.Num());
    Mips->BulkData.Unlock();
#else
    const TArray<uint8>* RawData = nullptr;
    ImageWrapper->SetCompressed(Content.GetData(), Content.Num());
    ImageWrapper->GetRaw(ERGBFormat::BGRA, 8, RawData);
    if (!RawData)
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::GetButtonTextureFromURL - Failed to decompress image file"));
        return nullptr;
    }

    UTexture2D *ButtonTexture = UTexture2D::CreateTransient(ImageWrapper->GetWidth(), ImageWrapper->GetHeight());
    FTexture2DMipMap* Mips = &ButtonTexture->PlatformData->Mips[0];
    FMemory::Memcpy(Mips->BulkData.Lock(LOCK_READ_WRITE), (*RawData).GetData(), (*RawData).Num());
    Mips->BulkData.Unlock();
#endif
    ButtonTexture->UpdateResource();
    return ButtonTexture;
}

void UStoveSDKPopup::SetCloseButtonAsDefault()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("UStoveSDKPopup::GetButtonTextureFromURL - Set button image as default"));
    _CloseButton->SetButtonStyle(_CloseButtonStyles[_DefaultCloseButtonStyleType - 1]);
}

#undef LOCTEXT_NAMESPACE
