﻿#pragma once

#include "StoveSDKObject.h"
#include <vector>


void OnError(const StovePCError error)
{
    FStoveError Error{ static_cast<int>(error.functionType),
                       static_cast<int>(error.result),
                       FString(error.message),
                       error.externalError
    };

    FindStoveSDKObject()->OnError(Error);
}

void OnInitComplete()
{
    FindStoveSDKObject()->OnInitComplete();
}

void OnToken(const StovePCToken token)
{
    FStoveToken Token{ FString(token.accessToken) };

    FindStoveSDKObject()->OnToken(Token);
}

void OnUser(const StovePCUser user)
{
    FStoveUser User{ user.memberNo,
                     FString(WCHAR_TO_TCHAR(user.nickname)),
                     FString(user.gameUserId)
    };

    FindStoveSDKObject()->OnUser(User);
}

void OnOwnership(int size, StovePCOwnership* ownerships)
{
    std::vector<FStoveOwnership> OwnershipArray;

    for (int index = 0 ; index < size; index++, ownerships++)
    {
        FStoveOwnership Ownership{ FString(ownerships->gameId),
                                   ownerships->gameCode,
                                   ownerships->ownershipCode,
                                   ownerships->purchaseDate,
                                   ownerships->memberNo };


        OwnershipArray.push_back(Ownership);
    }

    FindStoveSDKObject()->OnOwnership(size, &OwnershipArray[0]);
}

void OnSetStat(const StovePCStatValue statValue)
{
    FStoveStatValue StatValue{ statValue.currentValue,
                               statValue.updated,
                               FString(statValue.errorMessage) };

    FindStoveSDKObject()->OnSetStat(StatValue);
}

void OnStat(const StovePCStat stat)
{
    FStoveStat Stat{ FString(stat.statFullId.gameId),
                     FString(stat.statFullId.statId),
                     stat.memberNo,
                     stat.currentValue,
                     stat.updatedAt };

    FindStoveSDKObject()->OnStat(Stat);
}

void OnAchievement(const StovePCAchievement achievement)
{
    FStoveAchievement Achievement{ FString(achievement.achievementId),
                                   FString(achievement.name),
                                   FString(achievement.description),
                                   FString(achievement.defaultImage),
                                   FString(achievement.achievedImage),
                                   FStoveAchievementCondition{ achievement.condition.goalValue,
                                                               FString(achievement.condition.valueOperation),
                                                               FString(achievement.condition.type) },
                                    achievement.value,
                                    FString(achievement.status) };

    FindStoveSDKObject()->OnAchievement(Achievement);
}

void OnAllAchievement(int size, StovePCAchievement* achievements)
{
    std::vector<FStoveAchievement> AchievementArray;

    for (int index = 0; index < size; index++, achievements++)
    {
        FStoveAchievement Achievement{ FString(achievements->achievementId),
                                       FString(achievements->name),
                                       FString(achievements->description),
                                       FString(achievements->defaultImage),
                                       FString(achievements->achievedImage),
                                       FStoveAchievementCondition{achievements->condition.goalValue,
                                                                  achievements->condition.valueOperation,
                                                                  achievements->condition.type },
                                       achievements->value,
                                       FString(achievements->status) };

        AchievementArray.push_back(Achievement);
    }

    FindStoveSDKObject()->OnAllAchievement(size, &AchievementArray[0]);
}

void OnRank(int size, StovePCRank* ranks, unsigned int rankTotalCount)
{
    std::vector<FStoveRank> RankArray;

    for (int index = 0; index < size; index++, ranks++)
    {
        FStoveRank Rank{ ranks->memberNo,
                         ranks->score,
                         ranks->rank,
                         ranks->nickname,
                         ranks->profileImage };

        RankArray.push_back(Rank);
    }

    FindStoveSDKObject()->OnRank(size, &RankArray[0], rankTotalCount);
}

void OnAutoPopup(int size, StovePCAutoPopup* autoPopups, int headerSize, StovePCPopupRequestHeader* headers)
{
    std::vector<FStoveAutoPopup> AutoPopupArray;

    for (int index = 0; index < size; index++, autoPopups++)
    {
        FStoveAutoPopup AutoPopup{ FString(autoPopups->origin),
                                   autoPopups->id,
                                   FString(autoPopups->url),
                                   FStovePopupControl {
                                        FStovePopupControlUI{
                                            FStovePopupControlUIVisible{ autoPopups->control.ui.visible.closeButton,
                                                                         autoPopups->control.ui.visible.navigationBar,
                                                                         autoPopups->control.ui.visible.backButton,
                                                                         autoPopups->control.ui.visible.forwardButton,
                                                                         autoPopups->control.ui.visible.refreshButton,
                                                                         autoPopups->control.ui.visible.homeButton,
                                                                         autoPopups->control.ui.visible.disallowedButton
                                            },
                                            autoPopups->control.ui.disallowedDay,
                                            FStovePopupControlUICloseButtonImage{
                                                FStovePopupControlUICloseButtonImageDetail{ autoPopups->control.ui.closeButtonImage.normal.fileUrl,
                                                                                            autoPopups->control.ui.closeButtonImage.normal.fileId
                                                },
                                                FStovePopupControlUICloseButtonImageDetail{ autoPopups->control.ui.closeButtonImage.pressed.fileUrl,
                                                                                            autoPopups->control.ui.closeButtonImage.pressed.fileId
                                                },
                                                autoPopups->control.ui.closeButtonImage.type
                                            }
                                        }   
                                   }
        };
        AutoPopupArray.push_back(AutoPopup);
    }

    std::vector<FStovePopupRequestHeader> HeaderArray;

    for (int index = 0; index < headerSize; index++, headers++)
    {
        FStovePopupRequestHeader Header{ FString(headers->name),
                                         FString(headers->value)
        };                              
 
        HeaderArray.push_back(Header);
    }

    FindStoveSDKObject()->OnAutoPopup(size, &AutoPopupArray[0], headerSize, &HeaderArray[0]);
}

void OnManualPopup(int size, StovePCManualPopup* manualPopups, int headerSize, StovePCPopupRequestHeader* headers)
{
    std::vector<FStoveManualPopup> ManualPopupArray;

    for (int index = 0; index < size; index++, manualPopups++)
    {
        FStoveManualPopup ManualPopup{ FString(manualPopups->origin),
                                       FString(manualPopups->resourceKey),
                                       manualPopups->id,
                                       FString(manualPopups->url),
                                        FStovePopupControl {
                                            FStovePopupControlUI{
                                                FStovePopupControlUIVisible{ manualPopups->control.ui.visible.closeButton,
                                                                             manualPopups->control.ui.visible.navigationBar,
                                                                             manualPopups->control.ui.visible.backButton,
                                                                             manualPopups->control.ui.visible.forwardButton,
                                                                             manualPopups->control.ui.visible.refreshButton,
                                                                             manualPopups->control.ui.visible.homeButton,
                                                                             manualPopups->control.ui.visible.disallowedButton},
                                                manualPopups->control.ui.disallowedDay,
                                                FStovePopupControlUICloseButtonImage{
                                                FStovePopupControlUICloseButtonImageDetail{ FString(manualPopups->control.ui.closeButtonImage.normal.fileUrl),
                                                                                            FString(manualPopups->control.ui.closeButtonImage.normal.fileId)
                                                },
                                                FStovePopupControlUICloseButtonImageDetail{ FString(manualPopups->control.ui.closeButtonImage.pressed.fileUrl),
                                                                                            FString(manualPopups->control.ui.closeButtonImage.pressed.fileId)
                                                },
                                                manualPopups->control.ui.closeButtonImage.type
                                            }
                                        }
                                   }
        };
        ManualPopupArray.push_back(ManualPopup);
    }

    std::vector<FStovePopupRequestHeader> HeaderArray;

    for (int index = 0; index < headerSize; index++, headers++)
    {
        FStovePopupRequestHeader Header{ FString(headers->name),
                                         FString(headers->value)
        };

        HeaderArray.push_back(Header);
    }

    FindStoveSDKObject()->OnManualPopup(size, &ManualPopupArray[0], headerSize, &HeaderArray[0]);
}

void OnNewsPopup(StovePCNewsPopup newsPopup, int headerSize, StovePCPopupRequestHeader* headers)
{
    FStoveNewsPopup NewsPopup{ FString(newsPopup.origin),
                               newsPopup.id,
                               FString(newsPopup.url),
                                    FStovePopupControl {
                                    FStovePopupControlUI{
                                        FStovePopupControlUIVisible{ newsPopup.control.ui.visible.closeButton,
                                                                     newsPopup.control.ui.visible.navigationBar,
                                                                     newsPopup.control.ui.visible.backButton,
                                                                     newsPopup.control.ui.visible.forwardButton,
                                                                     newsPopup.control.ui.visible.refreshButton,
                                                                     newsPopup.control.ui.visible.homeButton,
                                                                     newsPopup.control.ui.visible.disallowedButton},
                                        newsPopup.control.ui.disallowedDay,
                                        FStovePopupControlUICloseButtonImage{
                                            FStovePopupControlUICloseButtonImageDetail{ FString(newsPopup.control.ui.closeButtonImage.normal.fileUrl),
                                                                                        FString(newsPopup.control.ui.closeButtonImage.normal.fileId)},
                                            FStovePopupControlUICloseButtonImageDetail{ FString(newsPopup.control.ui.closeButtonImage.pressed.fileUrl),
                                                                                        FString(newsPopup.control.ui.closeButtonImage.pressed.fileId)},
                                            newsPopup.control.ui.closeButtonImage.type
                                            }
                                        }
                                    }
    };

    std::vector<FStovePopupRequestHeader> HeaderArray;

    for (int index = 0; index < headerSize; index++, headers++)
    {
        FStovePopupRequestHeader Header{ FString(headers->name),
                                         FString(headers->value)
        };

        HeaderArray.push_back(Header);
    }

    FindStoveSDKObject()->OnNewsPopup( NewsPopup, headerSize, &HeaderArray[0]);
}

void OnCouponPopup(StovePCCouponPopup couponPopup, int headerSize, StovePCPopupRequestHeader* headers)
{
    FStoveCouponPopup CouponPopup{ FString(couponPopup.url),
                                   FStovePopupControl {
                                        FStovePopupControlUI{
                                            FStovePopupControlUIVisible{ couponPopup.control.ui.visible.closeButton,
                                                                     couponPopup.control.ui.visible.navigationBar,
                                                                     couponPopup.control.ui.visible.backButton,
                                                                     couponPopup.control.ui.visible.forwardButton,
                                                                     couponPopup.control.ui.visible.refreshButton,
                                                                     couponPopup.control.ui.visible.homeButton,
                                                                     couponPopup.control.ui.visible.disallowedButton},
                                            couponPopup.control.ui.disallowedDay,
                                            FStovePopupControlUICloseButtonImage{
                                                FStovePopupControlUICloseButtonImageDetail{ FString(couponPopup.control.ui.closeButtonImage.normal.fileUrl),
                                                                                            FString(couponPopup.control.ui.closeButtonImage.normal.fileId)},
                                                FStovePopupControlUICloseButtonImageDetail{ FString(couponPopup.control.ui.closeButtonImage.pressed.fileUrl),
                                                                                            FString(couponPopup.control.ui.closeButtonImage.pressed.fileId)},
                                                couponPopup.control.ui.closeButtonImage.type
                                            }
                                        }
                                   }
    };

    std::vector<FStovePopupRequestHeader> HeaderArray;

    for (int index = 0; index < headerSize; index++, headers++)
    {
        FStovePopupRequestHeader Header{ FString(headers->name),
                                         FString(headers->value)
        };

        HeaderArray.push_back(Header);
    }

    FindStoveSDKObject()->OnCouponPopup(CouponPopup, headerSize, &HeaderArray[0]);
}

void OnCommunityPopup(StovePCCommunityPopup communityPopup, int cookieSize, StovePCPopupRequestCookie* cookies)
{
    FStoveCommunityPopup CommunityPopup{ FString(communityPopup.url),
                                         FStovePopupControl {
                                            FStovePopupControlUI {
                                                FStovePopupControlUIVisible{ communityPopup.control.ui.visible.closeButton,
                                                                             communityPopup.control.ui.visible.navigationBar,
                                                                             communityPopup.control.ui.visible.backButton,
                                                                             communityPopup.control.ui.visible.forwardButton,
                                                                             communityPopup.control.ui.visible.refreshButton,
                                                                             communityPopup.control.ui.visible.homeButton,
                                                                             communityPopup.control.ui.visible.disallowedButton
                                                },
                                                communityPopup.control.ui.disallowedDay,
                                                FStovePopupControlUICloseButtonImage{
                                                    FStovePopupControlUICloseButtonImageDetail{ FString(communityPopup.control.ui.closeButtonImage.normal.fileUrl),
                                                                                                FString(communityPopup.control.ui.closeButtonImage.normal.fileId)
                                                    },
                                                    FStovePopupControlUICloseButtonImageDetail{ FString(communityPopup.control.ui.closeButtonImage.pressed.fileUrl),
                                                                                                FString(communityPopup.control.ui.closeButtonImage.pressed.fileId)},
                                                    communityPopup.control.ui.closeButtonImage.type
                                                }
                                            }
                                         }
    };

    std::vector<FStovePopupRequestCookie> CookieArray;
    for (int index = 0; index < cookieSize; index++, cookies++)
    {
        FStovePopupRequestCookie Cookie{
            FString(cookies->name),
            FString(cookies->value)
        };
        CookieArray.push_back(Cookie);
    }

    FindStoveSDKObject()->OnCommunityPopup(CommunityPopup, cookieSize, &CookieArray[0]);
}

void OnStashCustomEvent(const StovePCCustomEvent customEvent, int parameterSize, StovePCCustomEventParameter* parameters)
{
    FStoveCustomEvent CustomEvent{ FString(customEvent.name),
                                   FString(customEvent.category1),
                                   FString(customEvent.category2),
                                   customEvent.simpleValue
    };

    std::vector<FStoveCustomEventParameter> CustomEventParamArray;
    for (int index = 0; index < parameterSize; index++, parameters++)
    {
        FStoveCustomEventParameter Param{ FString(parameters->name),
                                          FString(parameters->value),

        };
        
        CustomEventParamArray.push_back(Param);
    }

    FindStoveSDKObject()->OnStashCustomEvent(CustomEvent, parameterSize, &CustomEventParamArray[0]);

}

void OnRenewToken(const StovePCToken token)
{
    FStoveToken Token { FString(token.accessToken) };
    FindStoveSDKObject()->OnRenewToken(Token);
}

void OnOverImmersion(const StovePCOverImmersion overImmersion)
{
    FStoveOverImmersion OverImmersion{ FString(overImmersion.message),
                                       overImmersion.elapsedTimeInHours,
                                       overImmersion.minExposureTimeInSeconds
    };
    
    FindStoveSDKObject()->OnOverImmersion(OverImmersion);
}

void OnShutdown(const StovePCShutdown shutdown)
{
    FStoveShutdown Shutdown{ shutdown.inadvanceTimeInMinutes,
                             FString(shutdown.message),
                             shutdown.exposureTimeInSeconds
    };
                           

    FindStoveSDKObject()->OnShutdown(Shutdown);
}


void OnFetchShopCategories(const int size, StovePCShopCategory* categories)
{
    std::vector<FStoveShopCategories> ShopCategoryArray;
    for (int index = 0; index < size; index++, categories++)
    {
        FStoveShopCategories Categories{ FString(categories->categoryId),
                                         FString(categories->parentCategoryId),
                                         categories->displayNo,
                                         FString(categories->name),
                                         categories->depth
        };

        ShopCategoryArray.push_back(Categories);
    }

    FindStoveSDKObject()->OnFetchShopCategories(size, &ShopCategoryArray[0]);
}

void OnFetchProducts(int size, StovePCProduct* products)
{
    std::vector<FStoveProduct> ProductArray;
    for (int index = 0; index < size; index++, products++)
    {
        FStoveProduct Product{ products->productId,
                                FString(products->gameItemId),
                                FString(products->name),
                                FString(products->description),
                                products->quantity,
                                products->productTypeCode,
                                FString(products->categoryId),
                                products->currencyCode,
                                products->price,
                                products->displayPrice,
                                FString(products->displayPriceString),
                                products->salePrice,
                                products->displaySalePrice,
                                FString(products->displaySalePriceString),
                                products->isDiscount,
                                products->discountType,
                                products->discountTypeValue,
                                products->discountBeginDate,
                                products->discountEndDate,
                                products->totalQuantity,
                                products->memberQuantity,
                                products->guidQuantity,
                                FString(products->thumbnailUrl),
                                products->canWithdraw
        };

        ProductArray.push_back(Product);
    }

    FindStoveSDKObject()->OnFetchProducts(size, &ProductArray[0]);
}

void OnStartPurchase(StovePCPurchase purchase)
{
    FStovePurchase Purchase{ purchase.transactionMasterNo,
                             FString(purchase.tempPaymentUrl),
                             purchase.purchaseProgress
    };

    FindStoveSDKObject()->OnStartPurchase(Purchase);
}

void OnConfirmPurchase(const int size, StovePCPurchaseProduct* purchaseProducts, bool status, char* shopKey)
{
    std::vector<FStovePurchaseProduct> PurchaseProductArray;
    for (int index = 0; index < size; index++, purchaseProducts++)
    {
        FStovePurchaseProduct PurchaseProduct{ purchaseProducts->productId,
                                               FString(purchaseProducts->categoryId),
                                               purchaseProducts->totalQuantity,
                                               purchaseProducts->memberQuantity,
                                               purchaseProducts->guidQuantity,
        };

        PurchaseProductArray.push_back(PurchaseProduct);
    }

    FindStoveSDKObject()->OnConfirmPurchase(size, &PurchaseProductArray[0], status, FString(shopKey));
}

void OnFetchInventory(const int size, StovePCInventoryItem* inventoryItems)
{
    std::vector<FStoveInventoryItem> InventoryArray;
    for (int index = 0; index < size; index++, inventoryItems++)
    {
        FStoveInventoryItem InventoryItem{ inventoryItems->transactionMasterNo,
                                           inventoryItems->transactionDetailNo,
                                           inventoryItems->productId,
                                           FString(inventoryItems->gameItemId),
                                           FString(inventoryItems->productName),
                                           inventoryItems->quantity,
                                           FString(inventoryItems->thumbnailUrl)
        };

        InventoryArray.push_back(InventoryItem);
    }

    FindStoveSDKObject()->OnFetchInventory(size, &InventoryArray[0]);
}

void OnFetchTermsAgreement(const StovePCTermsAgreement termsAgreement)
{
    FStoveTermsAgreement TermsAgreement { termsAgreement.gameId,
                                          termsAgreement.result,
                                          termsAgreement.region,
                                          termsAgreement.agreementUrl

    };
   
    FindStoveSDKObject()->OnFetchTermsAgreement(TermsAgreement);
}

