﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once
#include <memory>

#include "CoreMinimal.h"

#include "StoveSDKEnum.h"
#include "StoveSDKStruct.h"
#include "StoveSDKPlugin.h"
#include "StoveSDKPopup.h"
#include "StoveSDKWebInterface.h"
#include "StoveSDKObject.generated.h"

/**
 * 
 */
UCLASS()
class STOVESDKPLUGIN_API UStoveSDKObject : public UObject, 
                                           public FTickableGameObject
{
GENERATED_BODY()

public:
    UStoveSDKObject();
   
    // Begin FTickableGameObject Interface.
    virtual void Tick(float DeltaTime) override;
    virtual bool IsTickable() const override;
    virtual TStatId GetStatId() const override;
    // End FTickableGameObject Interface.


public:
    /* StovePCSDK API */
    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKInit(const FStoveConfig& Config);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKIAPInit(const FString& ShopKey);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKUnInit();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveState StoveSDKGetInitState();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetToken();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetUser();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetOwnership();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetStat(const FString& StatId);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKSetStat(const FString& StatId, const int StatValue);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetAchievement(const FString& AchievementId);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetAllAchievement();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetRank(const FString& LeaderboardId,
                                             const int PageIndex,
                                             const int PageSize,
                                             bool IncludeMyRank);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKSetGameProfile(const FString& WorldId, const FString& CharactorNo);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetAutoPopup();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetManualPopup(const FString& ResourceKey);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetNewsPopup();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetCouponPopup();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKGetCommunityPopup();

   UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FString StoveSDKGetSDKVersion();

   UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKStashCustomEvent(const FString& Name, 
                                                      const FString& Category1, 
                                                      const FString& Category2, 
                                                      const FString& SimpleValue, 
                                                      const TArray<FStoveCustomEventParameter> Params, 
                                                      int ParamSize);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKSetLanguage(const FString& Language);

   UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKSetPopupDisallowed(const int PopupId, const int Days );

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual const FString StoveSDKTranslateLanguage(const FString& Original);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual const FStoveTraceHint StoveSDKGetTraceHint();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKFetchTermsAgreement();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKFetchShopCategories();

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKFetchProducts(const FString& CategoryId, const bool IsRefresh);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKStartPurchase(const TArray<FStoveOrderProduct> Products, const int ProductSize);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKConfirmPurchase(const FString& TransactionMasterNo);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual const FStovePurchaseVerification StoveSDKVerifyPurchase(const FString& CategoryId, const FString& ProductId);

    UFUNCTION(BlueprintCallable, Category = "StoveSDKPlugin")
        virtual FStoveResult StoveSDKFetchInventory();
    /* StovePCSDK API */

    /* StovePCSDK Event */
    virtual void OnError(FStoveError Error) {}
    virtual void OnInitComplete() {}
    virtual void OnToken(FStoveToken Token) {}
    virtual void OnUser(FStoveUser User) {}
    virtual void OnOwnership(int Size, FStoveOwnership* Ownerships) {}

    virtual void OnStat(FStoveStat Stat) {}
    virtual void OnSetStat(FStoveStatValue StatValue) {}
    virtual void OnAchievement(FStoveAchievement Achievement) {}
    virtual void OnAllAchievement(int Size, FStoveAchievement* Achievements) {}
    virtual void OnRank(int Size, FStoveRank* Ranks, unsigned int RankTotalCount) {}
    
    virtual void OnAutoPopup(int Size, FStoveAutoPopup* AutoPopups, int HeaderSize, FStovePopupRequestHeader* Headers) {}
    virtual void OnManualPopup(int Size, FStoveManualPopup* ManualPopup, int HeaderSize, FStovePopupRequestHeader* Headers) {}
    virtual void OnNewsPopup(FStoveNewsPopup NewsPopup, int HeaderSize, FStovePopupRequestHeader* Headers) {}
    virtual void OnCouponPopup(FStoveCouponPopup CouponPopup, int HeaderSize, FStovePopupRequestHeader* Headers) {}
    virtual void OnCommunityPopup(FStoveCommunityPopup CommunityPopup, int CookieSize, FStovePopupRequestCookie* Cookies) {}

    virtual void OnStashCustomEvent(FStoveCustomEvent CustomEvent, int ParameterSize, FStoveCustomEventParameter* Parameters) {}
    virtual void OnRenewToken(FStoveToken Token) {}
    virtual void OnOverImmersion(FStoveOverImmersion OverImmersion) {}
    virtual void OnShutdown(FStoveShutdown) {}

    virtual void OnFetchShopCategories(int Size, FStoveShopCategories* Categories) {}
    virtual void OnFetchProducts(int Size, FStoveProduct* Products) {}
    virtual void OnStartPurchase(FStovePurchase Purchase) {}
    virtual void OnConfirmPurchase(int Size, FStovePurchaseProduct* PurchaseProducts, bool Status, FString ShopKey) {}
    virtual void OnFetchInventory(int Size, FStoveInventoryItem* InventoryItems) {}
    virtual void OnFetchTermsAgreement(const FStoveTermsAgreement TermsAgreement) {};
    /* StovePCSDK Event */

public:
    //StoveSDK Sample
    void SetCheckBoxState(const TArray<ECheckBoxState>& CallbackCheck); // only StoveSample Use

protected:
    virtual const TSharedPtr<IStoveSDKWebInterface> GetStoveSDKWeb() { return nullptr; }
    UPROPERTY()
    class UStoveSDKPopup *_PopupWidget = nullptr;

    bool _MakePopupWidget(const FString &ApiName);
    void _SetPopupAndDisplay(const FString &ApiName, 
                            TArray<FPopupUIFrame> PopupFrames, 
                            const TArray<FStovePopupRequestHeader>& Headers, 
                            const TArray<FStoveCefCookie>& Cookies);

    void _SetPopupAndDisplayNoFrame(const FString &ApiName,
                                    TArray<FPopupUIFrame> PopupFrames,
                                    const TArray<FStovePopupRequestHeader>& Headers,
                                    const TArray<FStoveCefCookie>& Cookies,
                                    const EStoveAlignment& Align);
};

inline UStoveSDKObject* FindStoveSDKObject()
{
    for (TObjectIterator<UStoveSDKObject> It; It; ++It)
    {
        return *It;
    }
    return nullptr;
}
