﻿// Copyright 1998-2019 Epic Games, Inc. All Rights Reserved.

#pragma once

#include "StoveSDKVersion.h"
#include "../../../Binaries/Thirdparty/StoveSDKPluginLibrary/include/StovePCSDK.h"
#include "Modules/ModuleManager.h"

class FStoveSDKPluginModule : public IModuleInterface
{
public:

    /** IModuleInterface implementation */
    virtual void StartupModule() override;
    virtual void ShutdownModule() override;

public:
    /** StovePCSDK Interface */
    StovePCResult                   Init(const StovePCConfig& config, const StovePCCallback& callback);
    StovePCResult                   IAPInit(const char* shopKey);
    StovePCResult                   UnInit();

    StovePCInitState                InitState();
    StovePCResult                   RunCallback();
    StovePCResult                   GetToken();
    StovePCResult                   GetUser();
    StovePCResult                   GetOwnership();
    StovePCResult                   GetStat(const char* statId);
    StovePCResult                   SetStat(const char* statId, const int statValue);
    StovePCResult                   GetAchievement(const char* achievementId);
    StovePCResult                   GetAllAchievement();
    StovePCResult                   GetRank(const char* leaderboardId,
                                            const unsigned int pageIndex,
                                            const unsigned int pageSize,
                                            bool includeMyRank);
    StovePCResult                   SetGameProfile(const char* worldId, const __int64 charactorNo);
    StovePCResult                   GetAutoPopup();
    StovePCResult                   GetManualPopup(const char* resourceKey);
    StovePCResult                   GetNewsPopup();
    StovePCResult                   GetCouponPopup();
    StovePCResult                   GetCommunityPopup();
    FString                         GetSDKVersion();
    StovePCResult                   StashCustomEvent(const char* name,
                                                     const char* category1,
                                                     const char* category2,
                                                     float simpleValue,
                                                     const StovePCCustomEventParameter* params,
                                                     int paramsSize);
    StovePCResult                   SetLanguage(const char* language);
    StovePCResult                   SetPopupDisallowed(const int popupId, const int disallowDays);
    wchar_t*                        TranslateLanguage(const char* stringId);
    StovePCTraceHint                GetTraceHint();
    StovePCResult                   FetchTermAgreement();
    StovePCResult                   FetchShopCategories();
    StovePCResult                   FetchProducts(const char* categoryId, const bool isRefresh);
    StovePCResult                   StartPurchase(const StovePCOrderProduct* products, int productsSize);
    StovePCResult                   ConfirmPurchase(const __int64 transactionMasterNo);
    StovePCPurchaseVerification     VerifyPurchase(const char* categoryId, const __int64 productId);
    StovePCResult                   FetchInventory();
    /** StovePCSDK Interface */
private:
    /** Handle to the StovePCSDK dll we will load */
    void*   _StoveSDKLibraryHandle;
};

inline FStoveSDKPluginModule& GetStoveSDKModule()
{
    return FModuleManager::Get().GetModuleChecked<FStoveSDKPluginModule>("StoveSDKPlugin");
}

DECLARE_LOG_CATEGORY_EXTERN(LogStoveSDKPlugin, Log, All);