﻿#pragma once

#include "CoreMinimal.h"
#include "Modules/ModuleManager.h"
#include "Engine.h"
#include "../../../Binaries/Thirdparty/StoveSDKPluginLibrary/include/StovePCDefine.h"
#include "StoveSDKStruct.generated.h"

USTRUCT(BlueprintType)
struct FStoveResult
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int Result;
};

USTRUCT(BlueprintType)
struct FStoveError
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int FunctionType;

    UPROPERTY()
        FStoveResult ErrorResult;

    UPROPERTY()
        FString Message;

    UPROPERTY()
        int ExternalError;
};

USTRUCT(BlueprintType)
struct FStoveConfig
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString Env;

    UPROPERTY()
        FString AppKey;

    UPROPERTY()
        FString SecretKey;

    UPROPERTY()
        FString GameId;

    UPROPERTY()
        int LogLevel;

    UPROPERTY()
        FString LogPath;
};

USTRUCT(BlueprintType)
struct FStoveToken
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString AccessToken;
};

USTRUCT(BlueprintType)
struct FStoveUser
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        uint64 MemberNo;

    UPROPERTY()
        FString Nickname;

    UPROPERTY()
        FString GameUserId;
};

USTRUCT(BlueprintType)
struct FStoveOwnership
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString GameId;

    UPROPERTY()
        int GameCode;

    UPROPERTY()
        int OwnershipCode;

    UPROPERTY()
        uint64 PurchaseDate;

    UPROPERTY()
        uint64 MemberNo;
};



USTRUCT(BlueprintType)
struct FStoveState
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int State;
};

USTRUCT(BlueprintType)
struct FStoveStatFullId
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString GameId;

    UPROPERTY()
        FString StatId;
};

USTRUCT(BlueprintType)
struct FStoveStat
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FStoveStatFullId StatFullId;

    UPROPERTY()
        int CurrentValue;

    UPROPERTY()
        uint64 MemberNo;

    UPROPERTY()
        uint64 UpdatedAt;
};

USTRUCT(BlueprintType)
struct FStoveStatValue
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int CurrentValue;

    UPROPERTY()
        bool bUpdated;

    UPROPERTY()
        FString ErrorMessage;
};

USTRUCT(BlueprintType)
struct FStoveAchievementCondition
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int GoalValue;

    UPROPERTY()
        FString ValueOperation;

    UPROPERTY()
        FString Type;
};

USTRUCT(BlueprintType)
struct FStoveAchievement
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString AchievementId;

    UPROPERTY()
        FString Name;

    UPROPERTY()
        FString Description;

    UPROPERTY()
        FString DefaultImage;

    UPROPERTY()
        FString AchievedImage;

    UPROPERTY()
        FStoveAchievementCondition Condition;

    UPROPERTY()
        int Value;

    UPROPERTY()
        FString Status;
};

USTRUCT(BlueprintType)
struct FStoveRank
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        uint64 MemberNo;

    UPROPERTY()
        int Score;

    UPROPERTY()
        unsigned int Rank;

    UPROPERTY()
        FString Nickname;

    UPROPERTY()
        FString ProfileImage;
};

USTRUCT(BlueprintType)
struct FStovePopupControlUIVisible
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        bool CloseButton;

    UPROPERTY()
        bool NavigationBar;

    UPROPERTY()
        bool BackButton;

    UPROPERTY()
        bool ForwardButton;

    UPROPERTY()
        bool RefreshButton;

    UPROPERTY()
        bool HomeButton;

    UPROPERTY()
        bool DisallowedButton;
};

USTRUCT(BlueprintType)
struct FStovePopupControlUICloseButtonImageDetail
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString FileUrl;

    UPROPERTY()
        FString FileId;
};

USTRUCT(BlueprintType)
struct FStovePopupControlUICloseButtonImage
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FStovePopupControlUICloseButtonImageDetail Normal;

    UPROPERTY()
        FStovePopupControlUICloseButtonImageDetail Pressed;

    UPROPERTY()
        int Type;
};


USTRUCT(BlueprintType)
struct FStovePopupControlUI
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FStovePopupControlUIVisible Visible;

    UPROPERTY()
        int DisallowedDay;

    UPROPERTY()
        FStovePopupControlUICloseButtonImage CloseButtonImage;
};


USTRUCT(BlueprintType)
struct FStovePopupControl
{
    GENERATED_USTRUCT_BODY()
    
    UPROPERTY()
        FStovePopupControlUI UI;
};


USTRUCT(BlueprintType)
struct FStoveAutoPopup
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString Origin;

    UPROPERTY()
        int Id;

    UPROPERTY()
        FString Url;

    UPROPERTY()
        FStovePopupControl Control;
};

USTRUCT(BlueprintType)
struct FStoveManualPopup
{
    GENERATED_USTRUCT_BODY() 

    UPROPERTY()
        FString Origin;

    UPROPERTY()
        FString ResourceKey;

    UPROPERTY()
        int Id;

    UPROPERTY()
        FString Url;

    UPROPERTY()
        FStovePopupControl Control;
};

USTRUCT(BlueprintType)
struct FStoveNewsPopup
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString Origin;

    UPROPERTY()
        int Id;

    UPROPERTY()
        FString Url;

    UPROPERTY()
        FStovePopupControl Control;
};

USTRUCT(BlueprintType)
struct FStoveCouponPopup
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString Url;

    UPROPERTY()
        FStovePopupControl Control;
};

USTRUCT(BlueprintType)
struct FStoveCommunityPopup
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString Url;

    UPROPERTY()
        FStovePopupControl Control;
};


USTRUCT(BlueprintType)
struct FStovePopupRequestHeader
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
    FString Name;

    UPROPERTY()
    FString Value;
};


USTRUCT(BlueprintType)
struct FStovePopupRequestCookie
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString Name;

    UPROPERTY()
        FString Value;
};

USTRUCT(BlueprintType)
struct FStoveCustomEventParameter
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString Name;

    UPROPERTY()
        FString Value;
};

USTRUCT(BlueprintType)
struct FStoveCustomEvent
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString Name;

    UPROPERTY()
        FString Category1;

    UPROPERTY()
        FString Category2;

    UPROPERTY()
        float SimpleValue;
};



USTRUCT(BlueprintType)
struct FStoveOverImmersion
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString Message;

    UPROPERTY()
        int ElapsedTimeInHours;

    UPROPERTY()
        int MinExposureTimeInSeconds;
};

USTRUCT(BlueprintType)
struct FStoveShutdown
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int InAdvanceTimeInMinutes;

    UPROPERTY()
        FString Message;

    UPROPERTY()
        int ExposureTimeInSeconds;
};

USTRUCT(BlueprintType)
struct FStoveTraceHint
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FStoveResult ErrorResult;

    UPROPERTY()
        FString SessionId;

    UPROPERTY()
        FString RefSessionId;
    
    UPROPERTY()
        FString Uuid;

    UPROPERTY()
        FString ServiceProtocol;

    UPROPERTY()
        FString RefSourceType;
};

USTRUCT(BlueprintType)
struct FStoveShopCategories
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString CategoryId;

    UPROPERTY()
        FString ParentCategoryId;

    UPROPERTY()
        int DisplayNo;

    UPROPERTY()
        FString Name;

    UPROPERTY()
        int Depth;
};


USTRUCT(BlueprintType)
struct FStoveProduct
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int64 ProductId;

    UPROPERTY()
        FString GameItemId;

    UPROPERTY()
        FString Name;

    UPROPERTY()
        FString Description; 

    UPROPERTY()
        int Quantity;

    UPROPERTY()
        uint16 ProductTypeCode;

    UPROPERTY()
        FString CategoryId;

    UPROPERTY()
        FString CurrencyCode;

    UPROPERTY()
        double Price;

    UPROPERTY()
        double DisplayPrice;

    UPROPERTY()
        FString DisplayPriceString;

    UPROPERTY()
        double SalePrice;

    UPROPERTY()
        double DisplaySalePrice;

    UPROPERTY()
        FString DisplaySalePriceString;

    UPROPERTY()
        bool bIsDiscount;

    UPROPERTY()
        uint16 DiscountType;

    UPROPERTY()
        int DiscountTypeValue;

    UPROPERTY()
        uint64 DiscountBeginDate;

    UPROPERTY()
        uint64 DiscountEndDate;

    UPROPERTY()
        int TotalQuantity;

    UPROPERTY()
        int MemberQuantity;

    UPROPERTY()
        int GuidQuantity;

    UPROPERTY()
        FString ThumbnailUrl;

    UPROPERTY()
        bool bCanWithdraw;
};

USTRUCT(BlueprintType)
struct FStoveOrderProduct
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int64 ProductId;

    UPROPERTY()
        double SalePrice;

    UPROPERTY()
        int32 Quantity;
};

USTRUCT(BlueprintType)
struct FStovePurchase
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int64 TransactionMasterNo;

    UPROPERTY()
        FString TempPaymentUrl;

    UPROPERTY()
        int PurchaseProgress;
};

USTRUCT(BlueprintType)
struct FStovePurchaseProduct
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int64 ProductId;

    UPROPERTY()
        FString CategoryId;

    UPROPERTY()
        int TotalQuantity;

    UPROPERTY()
        int MemberQuantity;

    UPROPERTY()
        int GuidQuantity;
};

USTRUCT(BlueprintType)
struct FStovePurchaseVerification
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FStoveResult ErrorResult;

    UPROPERTY()
        bool IsPurchased;
};

USTRUCT(BlueprintType)
struct FStoveInventoryItem
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        int64 TransactionMasterNo;

    UPROPERTY()
        int64 TransactionDetailNo;

    UPROPERTY()
        int64 ProductId;

    UPROPERTY()
        FString GameItemId;

    UPROPERTY()
        FString ProductName;

    UPROPERTY()
        int Quantity;

    UPROPERTY()
        FString ThumbnailUrl;
};

USTRUCT(BlueprintType)
struct FStoveTermsAgreement
{
    GENERATED_USTRUCT_BODY()

    UPROPERTY()
        FString GameId;

    UPROPERTY()
        bool Result;

    UPROPERTY()
        FString Region;

    UPROPERTY()
        FString AgreementUrl;
};



